#include "itkNumericTraits.h"

namespace itk
{
namespace Functor
{

// MIPt (T1 weighted measurement)
template <class TInput, class TOutput>
class CalcMax
{
public:

  typedef typename NumericTraits<TInput>::ValueType ValueType;

  CalcMax() {}
  ~CalcMax() {}

  inline TOutput operator()(const Array<TInput> &A,const Array<double> &T)
  {
    ValueType result=A[0];

    for (unsigned int i=1; i<A.size(); i++)
    {
      ValueType temp=A[i];
      if (temp>result) result=temp;
    }

    return static_cast<TOutput>(result);
  }
};

// MIPt (T2* weighted measurement)
template <class TInput, class TOutput>
class CalcMin
{
public:

  typedef typename NumericTraits<TInput>::ValueType ValueType;

  CalcMin() {}
  ~CalcMin() {}

  inline TOutput operator()(const Array<TInput> &A,const Array<double> &T)
  {
    ValueType result=A[0];

    for (unsigned int i=1; i<A.size(); i++)
    {
      ValueType temp=A[i];
      if (temp<result) result=temp;
    }

    return static_cast<TOutput>(result);
  }
};

// WI (T1 weighted measurement)
template <class TInput, class TOutput>
class CalcMaxDev
{
public:

  typedef typename NumericTraits<TInput>::FloatType FloatType;

  CalcMaxDev() {}
  ~CalcMaxDev() {}

  inline TOutput operator()(const Array<TInput> &A,const Array<double> &T)
  {
    FloatType result=0.0f;

    for (unsigned int i=1; i<A.size(); i++)
    {
      FloatType diff=static_cast<FloatType>(A[i])-static_cast<FloatType>(A[i-1]);
      double delta=T[i]-T[i-1];

      if (delta<=0.0f) delta=1.0;
      FloatType dev=diff/delta;
      if (dev>result) result=dev;
    }

    return static_cast<TOutput>(result);
  }
};

// WI (T2* weighted measurement)
template <class TInput, class TOutput>
class CalcMinDev
{
public:

  typedef typename NumericTraits<TInput>::FloatType FloatType;

  CalcMinDev() {}
  ~CalcMinDev() {}

  inline TOutput operator()(const Array<TInput> &A,const Array<double> &T)
  {
    FloatType result=0.0f;

    for (unsigned int i=1; i<A.size(); i++)
    {
      FloatType diff=static_cast<FloatType>(A[i])-static_cast<FloatType>(A[i-1]);
      double delta=T[i]-T[i-1];

      if (delta<=0.0f) delta=1.0;
      FloatType dev=diff/delta;
      if (dev<result) result=dev;
    }

    return static_cast<TOutput>(-result);
  }
};

}
}
